@@ -1,11 +1,11 @@
 function PosteriorIRF(type)
-% Builds posterior IRFs after the MH algorithm. 
-% 
-% INPUTS 
+% Builds posterior IRFs after the MH algorithm.
+%
+% INPUTS
 %   o type       [char]     string specifying the joint density of the
-%                           deep parameters ('prior','posterior'). 
-%  
-% OUTPUTS 
+%                           deep parameters ('prior','posterior').
+%
+% OUTPUTS
 %   None                    (oo_ and plots).
 %
 % SPECIAL REQUIREMENTS
@@ -14,9 +14,9 @@ function PosteriorIRF(type)
 % PARALLEL CONTEXT
 % This funtion has been parallelized in two different points. Then we have two core
 % functions associated with it(the _core1 and _core2).
-% See also the comments random_walk_metropolis_hastings.m funtion.
+% See also the comments posterior_sampler.m funtion.
 
-% Copyright (C) 2006-2013 Dynare Team
+% Copyright (C) 2006-2017 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -34,15 +34,16 @@ function PosteriorIRF(type)
 % along with Dynare.  If not, see <http://www.gnu.org/licenses/>.
 
 
-global options_ estim_params_ oo_ M_ bayestopt_ dataset_
+global options_ estim_params_ oo_ M_ bayestopt_ dataset_ dataset_info
+
 % Set the number of periods
-if isempty(options_.irf) || ~options_.irf 
+if isempty(options_.irf) || ~options_.irf
     options_.irf = 40;
 end
 % Set varlist if necessary
 varlist = options_.varlist;
 if isempty(varlist)
-    varlist = options_.varobs;
+    varlist = char(options_.varobs);
 end
 options_.varlist = varlist;
 nvar = size(varlist,1);
@@ -58,7 +59,7 @@ end
 
 % Get index of shocks for requested IRFs
 irf_shocks_indx = getIrfShocksIndx();
-    
+
 % Set various parameters & Check or create directories
 nvx  = estim_params_.nvx;
 nvn  = estim_params_.nvn;
@@ -68,8 +69,8 @@ np   = estim_params_.np ;
 npar = nvx+nvn+ncx+ncn+np;
 offset = npar-np; clear('nvx','nvn','ncx','ncn','np');
 
-nvobs = dataset_.info.nvobs;
-gend = dataset_.info.ntobs;
+nvobs = dataset_.vobs;
+gend = dataset_.nobs;
 MaxNumberOfPlotPerFigure = 9;
 nn = sqrt(MaxNumberOfPlotPerFigure);
 MAX_nirfs_dsge = ceil(options_.MaxNumberOfBytes/(options_.irf*nvar*M_.exo_nbr)/8);
@@ -92,12 +93,21 @@ elseif strcmpi(type,'gsa')
 else
     MhDirectoryName = CheckPath('prior',M_.dname);
 end
-delete([MhDirectoryName filesep M_.fname '_IRF_DSGEs*.mat']);
-delete([MhDirectoryName filesep M_.fname '_IRF_BVARDSGEs*.mat']);
+
+%delete old stale files before creating new ones
+delete_stale_file([MhDirectoryName filesep M_.fname '_IRF_DSGEs*.mat']);
+delete_stale_file([MhDirectoryName filesep M_.fname '_IRF_BVARDSGEs*.mat']);
+delete_stale_file([MhDirectoryName filesep M_.fname '_irf_dsge*.mat']);
+delete_stale_file([MhDirectoryName filesep M_.fname '_irf_bvardsge*.mat']);
+delete_stale_file([MhDirectoryName filesep M_.fname '_param_irf*.mat']);
+
+
 if strcmpi(type,'posterior')
-    load_last_mh_history_file(MhDirectoryName, M_.fname);
-    TotalNumberOfMhDraws = sum(record.MhDraws(:,1));
-    NumberOfDraws = TotalNumberOfMhDraws-floor(options_.mh_drop*TotalNumberOfMhDraws);
+    B = options_.sub_draws;
+    options_.B = B;
+    if round((1-options_.mh_conf_sig)*B)<2
+        fprintf('\nPosteriorIRF:: options_.mh_conf_sig times options_.sub_draws is too small to generate HPDIs. I am omitting them.\n')
+    end
 elseif strcmpi(type,'gsa')
     RootDirectoryName = CheckPath('gsa',M_.dname);
     if options_.opt_gsa.pprior
@@ -107,24 +117,12 @@ elseif strcmpi(type,'gsa')
     end
     x=[lpmat0(istable,:) lpmat(istable,:)];
     clear lpmat istable
-    NumberOfDraws=size(x,1);
-    B=NumberOfDraws; options_.B = B;
+    B=size(x,1); options_.B = B;
 else% type = 'prior'
-    NumberOfDraws = 500;
-end
-if ~strcmpi(type,'gsa')
-    B = min([round(.5*NumberOfDraws),500]); options_.B = B;
-end
-try 
-    delete([MhDirectoryName filesep M_.fname '_irf_dsge*.mat'])
-catch 
-    disp('No _IRFs (dsge) files to be deleted!')
-end
-try 
-    delete([MhDirectoryName filesep M_.fname '_irf_bvardsge*.mat'])
-catch 
-    disp('No _IRFs (bvar-dsge) files to be deleted!')
+    B = options_.prior_draws;
+    options_.B = B;
 end
+
 irun = 0;
 IRUN = 0;
 irun2 = 0;
@@ -148,10 +146,6 @@ if MAX_nirfs_dsgevar
     else
         stock_irf_bvardsge = zeros(options_.irf,nvobs,M_.exo_nbr,B);
     end
-    [mYY,mXY,mYX,mXX,Ydata,Xdata] = ...
-        var_sample_moments(options_.first_obs,options_.first_obs+options_.nobs-1,...
-                           options_.dsge_varlag,-1,options_.datafile,options_.varobs,...
-                           options_.xls_sheet,options_.xls_range);
     NumberOfLags = options_.dsge_varlag;
     NumberOfLagsTimesNvobs = NumberOfLags*nvobs;
     if options_.noconstant
@@ -178,13 +172,13 @@ localVars.npar = npar;
 
 localVars.type=type;
 if strcmpi(type,'posterior')
-    while b<B 
+    while b<B
         b = b + 1;
         x(b,:) = GetOneDraw(type);
     end
 end
 
-if ~strcmpi(type,'prior'),
+if ~strcmpi(type,'prior')
     localVars.x=x;
 end
 
@@ -196,7 +190,7 @@ if options_.dsge_var
     localVars.NumberOfLags = options_.dsge_varlag;
     localVars.NumberOfLagsTimesNvobs = NumberOfLags*nvobs;
     localVars.Companion_matrix = diag(ones(nvobs*(NumberOfLags-1),1),-nvobs);
-end 
+end
 localVars.nvar=nvar;
 localVars.IndxVariables=IndxVariables;
 localVars.MAX_nirfs_dsgevar=MAX_nirfs_dsgevar;
@@ -208,16 +202,16 @@ localVars.ifil2=ifil2;
 localVars.MhDirectoryName=MhDirectoryName;
 
 % Like sequential execution!
-if isnumeric(options_.parallel),
+if isnumeric(options_.parallel)
     [fout] = PosteriorIRF_core1(localVars,1,B,0);
     nosaddle = fout.nosaddle;
 else
     % Parallel execution!
     [nCPU, totCPU, nBlockPerCPU] = distributeJobs(options_.parallel, 1, B);
-    for j=1:totCPU-1,
+    for j=1:totCPU-1
         nfiles = ceil(nBlockPerCPU(j)/MAX_nirfs_dsge);
         NumberOfIRFfiles_dsge(j+1) =NumberOfIRFfiles_dsge(j)+nfiles;
-        if MAX_nirfs_dsgevar,
+        if MAX_nirfs_dsgevar
             nfiles = ceil(nBlockPerCPU(j)/MAX_nirfs_dsgevar);
         else
             nfiles=0;
@@ -229,32 +223,38 @@ else
     localVars.NumberOfIRFfiles_dsge=NumberOfIRFfiles_dsge;
     localVars.NumberOfIRFfiles_dsgevar=NumberOfIRFfiles_dsgevar;
     localVars.ifil2=ifil2;
-    
+
     globalVars = struct('M_',M_, ...
                         'options_', options_, ...
                         'bayestopt_', bayestopt_, ...
                         'estim_params_', estim_params_, ...
                         'oo_', oo_, ...
-                        'dataset_',dataset_);
-    
+                        'dataset_',dataset_, ...
+                        'dataset_info',dataset_info);
+
     % which files have to be copied to run remotely
     NamFileInput(1,:) = {'',[M_.fname '_static.m']};
     NamFileInput(2,:) = {'',[M_.fname '_dynamic.m']};
-    if options_.steadystate_flag,
-        NamFileInput(length(NamFileInput)+1,:)={'',[M_.fname '_steadystate.m']};
+    NamFileInput(3,:) = {'',[M_.fname '_set_auxiliary_variables.m']};
+    if options_.steadystate_flag
+        if options_.steadystate_flag == 1
+            NamFileInput(length(NamFileInput)+1,:)={'',[M_.fname '_steadystate.m']};
+        else
+            NamFileInput(length(NamFileInput)+1,:)={'',[M_.fname '_steadystate2.m']};
+        end
     end
     [fout] = masterParallel(options_.parallel, 1, B,NamFileInput,'PosteriorIRF_core1', localVars, globalVars, options_.parallel_info);
     nosaddle=0;
-    for j=1:length(fout),
+    for j=1:length(fout)
         nosaddle = nosaddle + fout(j).nosaddle;
     end
-    
+
 end
 
 % END first parallel section!
 
 if nosaddle
-    disp(['PosteriorIRF :: Percentage of discarded posterior draws = ' num2str(nosaddle/(B+nosaddle))]) 
+    disp(['PosteriorIRF :: Percentage of discarded posterior draws = ' num2str(nosaddle/(B+nosaddle))])
 end
 
 ReshapeMatFiles('irf_dsge',type)
@@ -312,12 +312,12 @@ clear STOCK_IRF_DSGE;
 for i = irf_shocks_indx
     for j = 1:nvar
         name = [deblank(M_.endo_names(IndxVariables(j),:)) '_' deblank(tit(i,:))];
-        eval(['oo_.PosteriorIRF.dsge.Mean.' name ' = MeanIRF(:,j,i);']);
-        eval(['oo_.PosteriorIRF.dsge.Median.' name ' = MedianIRF(:,j,i);']);
-        eval(['oo_.PosteriorIRF.dsge.Var.' name ' = VarIRF(:,j,i);']);
-        eval(['oo_.PosteriorIRF.dsge.deciles.' name ' = DistribIRF(:,:,j,i);']);
-        eval(['oo_.PosteriorIRF.dsge.HPDinf.' name ' = HPDIRF(:,1,j,i);']);
-        eval(['oo_.PosteriorIRF.dsge.HPDsup.' name ' = HPDIRF(:,2,j,i);']);
+        oo_.PosteriorIRF.dsge.Mean.(name) = MeanIRF(:,j,i);
+        oo_.PosteriorIRF.dsge.Median.(name) = MedianIRF(:,j,i);
+        oo_.PosteriorIRF.dsge.Var.(name) = VarIRF(:,j,i);
+        oo_.PosteriorIRF.dsge.deciles.(name) = DistribIRF(:,:,j,i);
+        oo_.PosteriorIRF.dsge.HPDinf.(name) = HPDIRF(:,1,j,i);
+        oo_.PosteriorIRF.dsge.HPDsup.(name) = HPDIRF(:,2,j,i);
     end
 end
 
@@ -327,7 +327,7 @@ if MAX_nirfs_dsgevar
     MedianIRFdsgevar = zeros(options_.irf,nvar,M_.exo_nbr);
     VarIRFdsgevar = zeros(options_.irf,nvar,M_.exo_nbr);
     DistribIRFdsgevar = zeros(options_.irf,9,nvar,M_.exo_nbr);
-    HPDIRFdsgevar = zeros(options_.irf,2,nvar,M_.exo_nbr);    
+    HPDIRFdsgevar = zeros(options_.irf,2,nvar,M_.exo_nbr);
     fprintf('Estimation::mcmc: Posterior (bvar-dsge) IRFs...\n');
     tit(M_.exo_names_orig_ord,:) = M_.exo_names;
     kdx = 0;
@@ -345,16 +345,16 @@ if MAX_nirfs_dsgevar
         end
         kdx = kdx + size(STOCK_IRF_BVARDSGE,1);
     end
-    clear STOCK_IRF_BVARDSGE; 
+    clear STOCK_IRF_BVARDSGE;
     for i = irf_shocks_indx
         for j = 1:nvar
             name = [deblank(M_.endo_names(IndxVariables(j),:)) '_' deblank(tit(i,:))];
-            eval(['oo_.PosteriorIRF.bvardsge.Mean.' name ' = MeanIRFdsgevar(:,j,i);']);
-            eval(['oo_.PosteriorIRF.bvardsge.Median.' name ' = MedianIRFdsgevar(:,j,i);']);
-            eval(['oo_.PosteriorIRF.bvardsge.Var.' name ' = VarIRFdsgevar(:,j,i);']);
-            eval(['oo_.PosteriorIRF.bvardsge.deciles.' name ' = DistribIRFdsgevar(:,:,j,i);']);
-            eval(['oo_.PosteriorIRF.bvardsge.HPDinf.' name ' = HPDIRFdsgevar(:,1,j,i);']);
-            eval(['oo_.PosteriorIRF.bvardsge.HPDsup.' name ' = HPDIRFdsgevar(:,2,j,i);']);
+            oo_.PosteriorIRF.bvardsge.Mean.(name) = MeanIRFdsgevar(:,j,i);
+            oo_.PosteriorIRF.bvardsge.Median.(name) = MedianIRFdsgevar(:,j,i);
+            oo_.PosteriorIRF.bvardsge.Var.(name) = VarIRFdsgevar(:,j,i);
+            oo_.PosteriorIRF.bvardsge.deciles.(name) = DistribIRFdsgevar(:,:,j,i);
+            oo_.PosteriorIRF.bvardsge.HPDinf.(name) = HPDIRFdsgevar(:,1,j,i);
+            oo_.PosteriorIRF.bvardsge.HPDsup.(name) = HPDIRFdsgevar(:,2,j,i);
         end
     end
 end
@@ -367,104 +367,103 @@ end
 % .tex generation always run in sequentially. This portion of code is execute in parallel by
 % PosteriorIRF_core2.m function.
 
+if ~options_.nograph && ~options_.no_graph.posterior
+    % Save the local variables.
+    localVars=[];
 
-% Save the local variables.
-localVars=[];
-
-Check=options_.TeX;
-if (Check)
-    localVars.varlist_TeX=varlist_TeX;
-end
+    Check=options_.TeX;
+    if (Check)
+        localVars.varlist_TeX=varlist_TeX;
+    end
 
 
-localVars.nvar=nvar;
-localVars.MeanIRF=MeanIRF;
-localVars.tit=tit;
-localVars.nn=nn;
-localVars.MAX_nirfs_dsgevar=MAX_nirfs_dsgevar;
-localVars.HPDIRF=HPDIRF;
-localVars.varlist=varlist;
-localVars.MaxNumberOfPlotPerFigure=MaxNumberOfPlotPerFigure;
-if options_.dsge_var
-    localVars.HPDIRFdsgevar=HPDIRFdsgevar;
-    localVars.MeanIRFdsgevar = MeanIRFdsgevar;
-end    
+    localVars.nvar=nvar;
+    localVars.MeanIRF=MeanIRF;
+    localVars.tit=tit;
+    localVars.nn=nn;
+    localVars.MAX_nirfs_dsgevar=MAX_nirfs_dsgevar;
+    localVars.HPDIRF=HPDIRF;
+    localVars.varlist=varlist;
+    localVars.MaxNumberOfPlotPerFigure=MaxNumberOfPlotPerFigure;
+    if options_.dsge_var
+        localVars.HPDIRFdsgevar=HPDIRFdsgevar;
+        localVars.MeanIRFdsgevar = MeanIRFdsgevar;
+    end
 
-% The files .TeX are genereted in sequential way always!
+    % The files .TeX are genereted in sequential way always!
 
-if options_.TeX
-    fidTeX = fopen([DirectoryName filesep M_.fname '_BayesianIRF.TeX'],'w');
-    fprintf(fidTeX,'%% TeX eps-loader file generated by PosteriorIRF.m (Dynare).\n');
-    fprintf(fidTeX,['%% ' datestr(now,0) '\n']);
-    fprintf(fidTeX,' \n');
-    titTeX(M_.exo_names_orig_ord,:) = M_.exo_names_tex;
-    
-    for i=irf_shocks_indx
-        NAMES = [];
-        TEXNAMES = [];
-        
-        for j=1:nvar
-            if max(abs(MeanIRF(:,j,i))) > options_.impulse_responses.plot_threshold  
-                
-                name = deblank(varlist(j,:));
-                texname = deblank(varlist_TeX(j,:));
+    % The files .TeX are generated in sequential way always!
+    subplotnum = 0;
+    tit_TeX(M_.exo_names_orig_ord,:) = M_.exo_names_tex;
+    if options_.TeX && any(strcmp('eps',cellstr(options_.graph_format)))
+        fidTeX = fopen([DirectoryName filesep M_.fname '_BayesianIRF.tex'],'w');
+        fprintf(fidTeX,'%% TeX eps-loader file generated by PosteriorIRF.m (Dynare).\n');
+        fprintf(fidTeX,['%% ' datestr(now,0) '\n']);
+        fprintf(fidTeX,' \n');
+        titTeX(M_.exo_names_orig_ord,:) = M_.exo_names_tex;
 
-                if j==1
-                    NAMES = name;
-                    TEXNAMES = ['$' texname '$'];
-                else
-                    NAMES = char(NAMES,name);
-                    TEXNAMES = char(TEXNAMES,['$' texname '$']);
+        for ii=irf_shocks_indx
+            figunumber = 0;
+
+            for jj=1:nvar
+                if max(abs(MeanIRF(:,jj,ii))) >= options_.impulse_responses.plot_threshold
+                    subplotnum = subplotnum+1;
+
+                    if subplotnum == 1
+                        fprintf(fidTeX,'\\begin{figure}[H]\n');
+                    end
+                    name = deblank(varlist(jj,:));
+                    texname = deblank(varlist_TeX(jj,:));
+                    fprintf(fidTeX,['\\psfrag{%s}[1][][0.5][0]{%s}\n'],name,['$' texname '$']);
+                end
+                if subplotnum == MaxNumberOfPlotPerFigure || (jj == nvar  && subplotnum> 0)
+                    figunumber = figunumber+1;
+
+                    fprintf(fidTeX,'\\centering \n');
+                    fprintf(fidTeX,'\\includegraphics[width=%2.2f\\textwidth]{%s/%s_Bayesian_IRF_%s_%d}\n',options_.figures.textwidth*min(subplotnum/nn,1),DirectoryName,M_.fname,deblank(tit(ii,:)),figunumber);
+                    if options_.relative_irf
+                        fprintf(fidTeX,['\\caption{Bayesian relative IRF.}']);
+                    else
+                        fprintf(fidTeX,'\\caption{Bayesian IRF: Orthogonalized shock to $%s$.}\n',deblank(tit_TeX(ii,:)));
+                    end
+                    fprintf(fidTeX,'\\label{Fig:BayesianIRF:%s:%d}\n',deblank(tit(ii,:)),figunumber);
+                    fprintf(fidTeX,'\\end{figure}\n');
+                    fprintf(fidTeX,' \n');
+
+                    subplotnum = 0;
                 end
             end
-            
         end
-        fprintf(fidTeX,'\\begin{figure}[H]\n');
-        for jj = 1:size(TEXNAMES,1)
-            fprintf(fidTeX,['\\psfrag{%s}[1][][0.5][0]{%s}\n'],deblank(NAMES(jj,:)),deblank(TEXNAMES(jj,:)));
-        end
-        fprintf(fidTeX,'\\centering \n');
-        fprintf(fidTeX,'\\includegraphics[scale=0.5]{%s_Bayesian_IRF_%s}\n',M_.fname,deblank(tit(i,:)));
-        if options_.relative_irf
-            fprintf(fidTeX,['\\caption{Bayesian relative IRF.}']);
-        else
-            fprintf(fidTeX,'\\caption{Bayesian IRF.}');
-        end
-        fprintf(fidTeX,'\\label{Fig:BayesianIRF:%s}\n',deblank(tit(i,:)));
-        fprintf(fidTeX,'\\end{figure}\n');
-        fprintf(fidTeX,' \n');
+        fprintf(fidTeX,'%% End of TeX file.\n');
+        fclose(fidTeX);
     end
-    
-    fprintf(fidTeX,'%% End of TeX file.\n');
-    fclose(fidTeX);
-    
-end
 
-% The others file format are generated in parallel by PosteriorIRF_core2!
+    % The others file format are generated in parallel by PosteriorIRF_core2!
 
 
-% Comment for testing!
-if ~isoctave
-    if isnumeric(options_.parallel)  || (M_.exo_nbr*ceil(size(varlist,1)/MaxNumberOfPlotPerFigure))<8,
-        [fout] = PosteriorIRF_core2(localVars,1,M_.exo_nbr,0);
-    else
-        isRemoteOctave = 0;
-        for indPC=1:length(options_.parallel),
-            isRemoteOctave = isRemoteOctave + (findstr(options_.parallel(indPC).MatlabOctavePath, 'octave'));
-        end
-        if isRemoteOctave
+    % Comment for testing!
+    if ~isoctave
+        if isnumeric(options_.parallel)  || (M_.exo_nbr*ceil(size(varlist,1)/MaxNumberOfPlotPerFigure))<8
             [fout] = PosteriorIRF_core2(localVars,1,M_.exo_nbr,0);
         else
-            globalVars = struct('M_',M_, ...
-                'options_', options_);
-            
-            [fout] = masterParallel(options_.parallel, 1, M_.exo_nbr,NamFileInput,'PosteriorIRF_core2', localVars, globalVars, options_.parallel_info);
+            isRemoteOctave = 0;
+            for indPC=1:length(options_.parallel)
+                isRemoteOctave = isRemoteOctave + (findstr(options_.parallel(indPC).MatlabOctavePath, 'octave'));
+            end
+            if isRemoteOctave
+                [fout] = PosteriorIRF_core2(localVars,1,M_.exo_nbr,0);
+            else
+                globalVars = struct('M_',M_, ...
+                                    'options_', options_);
+
+                [fout] = masterParallel(options_.parallel, 1, M_.exo_nbr,NamFileInput,'PosteriorIRF_core2', localVars, globalVars, options_.parallel_info);
+            end
         end
+    else
+        [fout] = PosteriorIRF_core2(localVars,1,M_.exo_nbr,0);
     end
-else
-    [fout] = PosteriorIRF_core2(localVars,1,M_.exo_nbr,0);
+    % END parallel code!
+
 end
-% END parallel code!
 
-
-fprintf('Estimation::mcmc: Posterior IRFs, done!\n');
\ No newline at end of file
+fprintf('Estimation::mcmc: Posterior IRFs, done!\n');
