@@ -1,5 +1,7 @@
 function [nvar,vartan,NumberOfDecompFiles] = ...
     dsge_simulated_theoretical_variance_decomposition(SampleSize,M_,options_,oo_,type)
+% function [nvar,vartan,NumberOfDecompFiles] = ...
+%     dsge_simulated_theoretical_variance_decomposition(SampleSize,M_,options_,oo_,type)
 % This function computes the posterior or prior distribution of the variance
 % decomposition of the observed endogenous variables.
 %
@@ -16,7 +18,7 @@ function [nvar,vartan,NumberOfDecompFiles] = ...
 %   vartan            [char]     array of characters (with nvar rows).
 %   CovarFileNumber   [integer]  scalar, number of prior or posterior data files (for covariance).
 
-% Copyright (C) 2007-2012 Dynare Team
+% Copyright (C) 2007-2017 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -47,7 +49,13 @@ else
     disp('dsge_simulated_theoretical_variance_decomposition:: Unknown type!')
     error()
 end
-NumberOfDrawsFiles = length(DrawsFiles);
+
+%delete old stale files before creating new ones
+if posterior
+    delete_stale_file([M_.dname '/metropolis/' M_.fname '_PosteriorVarianceDecomposition*']);
+else
+    delete_stale_file([M_.dname '/prior/moments/' M_.fname '_PosteriorVarianceDecomposition*']);
+end
 
 % Set varlist (vartan)
 if ~posterior
@@ -91,6 +99,7 @@ DecompFileNumber = 1;
 % Compute total variances (covariances are not saved) and variances
 % implied by each structural shock.
 linea = 0;
+only_non_stationary_vars=0;
 for file = 1:NumberOfDrawsFiles
     if posterior
         load([M_.dname '/metropolis/' DrawsFiles(file).name ]);
@@ -104,13 +113,29 @@ for file = 1:NumberOfDrawsFiles
         if isdrsaved
             dr = pdraws{linee,2};
         else
-            set_parameters(pdraws{linee,1});
+            M_=set_parameters_locally(M_,pdraws{linee,1});
             [dr,info,M_,options_,oo_] = resol(0,M_,options_,oo_);
         end
-        tmp = th_autocovariances(dr,ivar,M_,options_,nodecomposition);
-        for i=1:nvar
-            for j=1:nexo
-                Decomposition_array(linea,(i-1)*nexo+j) = tmp{2}(i,j);
+        if file==1 && linee==1
+            [tmp, stationary_vars] = th_autocovariances(dr,ivar,M_,options_,nodecomposition);
+            if isempty(stationary_vars)
+                fprintf('\ndsge_simulated_theoretical_variance_decomposition:: All requested endogenous variables have a unit root and thus infinite variance.\n')
+                fprintf('dsge_simulated_theoretical_variance_decomposition:: No decomposition is performed.\n')
+                only_non_stationary_vars=1;
+            end
+        end
+        if only_non_stationary_vars
+            for i=1:nvar
+                for j=1:nexo
+                    Decomposition_array(linea,(i-1)*nexo+j) = NaN;
+                end
+            end
+        else
+            tmp = th_autocovariances(dr,ivar,M_,options_,nodecomposition);
+            for i=1:nvar
+                for j=1:nexo
+                    Decomposition_array(linea,(i-1)*nexo+j) = tmp{2}(i,j);
+                end
             end
         end
         if linea == NumberOfDecompLines
@@ -125,7 +150,7 @@ for file = 1:NumberOfDrawsFiles
             if ~test% Prepare the last round...
                 Decomposition_array = zeros(NumberOfLinesInTheLastDecompFile,nvar*nexo);
                 NumberOfDecompLines = NumberOfLinesInTheLastDecompFile;
-            elseif test<0;
+            elseif test<0
                 Decomposition_array = zeros(MaXNumberOfDecompLines,nvar*nexo);
             else
                 clear('Decomposition_array');
