@@ -1,4 +1,4 @@
-function [dLIK, dlikk, a, Pstar, llik] = univariate_kalman_filter_d(data_index, number_of_observations, no_more_missing_observations, Y, start, last, a, Pinf, Pstar, kalman_tol, riccati_tol, presample, T, R, Q, H, Z, mm, pp, rr)
+function [dLIK, dlikk, a, Pstar, llik] = univariate_kalman_filter_d(data_index, number_of_observations, no_more_missing_observations, Y, start, last, a, Pinf, Pstar, kalman_tol, diffuse_kalman_tol, riccati_tol, presample, T, R, Q, H, Z, mm, pp, rr)
 % Computes the likelihood of a state space model (initialization with diffuse steps, univariate approach).
 
 %@info:
@@ -80,8 +80,14 @@ function [dLIK, dlikk, a, Pstar, llik] = univariate_kalman_filter_d(data_index,
 %! @ref{univariate_kalman_filter_ss}
 %! @end deftypefn
 %@eod:
+%
+% Algorithm:
+%
+%   Uses the diffuse univariate filter as described in Durbin/Koopman (2012): "Time
+%   Series Analysis by State Space Methods", Oxford University Press,
+%   Second Edition, Ch. 5, 6.4 + 7.2.5
 
-% Copyright (C) 2004-2012 Dynare Team
+% Copyright (C) 2004-2017 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -108,21 +114,28 @@ t    = start;              % Initialization of the time index.
 dlikk= zeros(smpl,1);      % Initialization of the vector gathering the densities.
 dLIK = Inf;                % Default value of the log likelihood.
 oldK = Inf;
-llik = NaN(smpl,pp);
+llik = zeros(smpl,pp);
 
-newRank = rank(Pinf,kalman_tol);
+newRank = rank(Pinf,diffuse_kalman_tol);
 l2pi = log(2*pi);
+s=0;
 
 while newRank && (t<=last)
     s = t-start+1;
     d_index = data_index{t};
     for i=1:length(d_index)
         Zi = Z(d_index(i),:);
-        prediction_error = Y(d_index(i),t) - Zi*a;
-        Fstar = Zi*Pstar*Zi' + H(d_index(i));
-        Finf  = Zi*Pinf*Zi';
+        prediction_error = Y(d_index(i),t) - Zi*a;      % nu_{t,i} in 6.13 in DK (2012)
+        Fstar = Zi*Pstar*Zi' + H(d_index(i));           % F_{*,t} in 5.7 in DK (2012), relies on H being diagonal
+        Finf  = Zi*Pinf*Zi';                            % F_{\infty,t} in 5.7 in DK (2012), relies on H being diagonal
         Kstar = Pstar*Zi';
-        if Finf>kalman_tol && newRank
+        % Conduct check of rank
+        % Pinf and Finf are always scaled such that their norm=1: Fstar/Pstar, instead,
+        % depends on the actual values of std errors in the model and can be badly scaled.
+        % experience is that diffuse_kalman_tol has to be bigger than kalman_tol, to ensure
+        % exiting the diffuse filter properly, avoiding tests that provide false non-zero rank for Pinf.
+        % Also the test for singularity is better set coarser for Finf than for Fstar for the same reason
+        if Finf>diffuse_kalman_tol && newRank           % F_{\infty,t,i} = 0, use upper part of bracket on p. 175 DK (2012) for w_{t,i}
             Kinf   = Pinf*Zi';
             Kinf_Finf = Kinf/Finf;
             a         = a + Kinf_Finf*prediction_error;
@@ -131,14 +144,17 @@ while newRank && (t<=last)
             llik(s,d_index(i)) = log(Finf) + l2pi;
             dlikk(s) = dlikk(s) + llik(s,d_index(i));
         elseif Fstar>kalman_tol
-            llik(s,d_index(i)) = log(Fstar) + prediction_error*prediction_error/Fstar + l2pi;
+            llik(s,d_index(i)) = log(Fstar) + (prediction_error*prediction_error/Fstar) + l2pi;
             dlikk(s) = dlikk(s) + llik(s,d_index(i));
             a = a+Kstar*(prediction_error/Fstar);
             Pstar = Pstar-Kstar*(Kstar'/Fstar);
+        else
+            % do nothing as a_{t,i+1}=a_{t,i} and P_{t,i+1}=P_{t,i}, see
+            % p. 157, DK (2012)
         end
     end
     if newRank
-        oldRank = rank(Pinf,kalman_tol);
+        oldRank = rank(Pinf,diffuse_kalman_tol);
     else
         oldRank = 0;
     end
@@ -146,7 +162,7 @@ while newRank && (t<=last)
     Pstar = T*Pstar*T'+QQ;
     Pinf  = T*Pinf*T';
     if newRank
-        newRank = rank(Pinf,kalman_tol);
+        newRank = rank(Pinf,diffuse_kalman_tol);
     end
     if oldRank ~= newRank
         disp('univariate_diffuse_kalman_filter:: T does influence the rank of Pinf!')
@@ -165,3 +181,4 @@ dlikk = .5*dlikk(1:s);
 llik  = .5*llik(1:s,:);
 
 dLIK = sum(dlikk(1+presample:end));
+dlikk = llik;
\ No newline at end of file
