@@ -1,6 +1,6 @@
 function info=stoch_simul(var_list)
 
-% Copyright (C) 2001-2013 Dynare Team
+% Copyright (C) 2001-2017 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -34,14 +34,8 @@ if options_.linear
 end
 if options_.order == 1
     options_.replic = 1;
-elseif options_.order == 3
-    options_.k_order_solver = 1;
 end
 
-% if options_.loglinear == 1 && options_.periods>0
-%    error('The loglinear option does not work with periods>0.')
-% end
-
 if isempty(options_.qz_criterium)
     options_.qz_criterium = 1+1e-6;
 end
@@ -62,7 +56,8 @@ if size(var_list,1) == 0
     var_list = M_.endo_names(1:M_.orig_endo_nbr, :);
 end
 
-[i_var,nvar] = varlist_indices(var_list,M_.endo_names);
+[i_var,nvar,index_uniques] = varlist_indices(var_list,M_.endo_names);
+var_list=var_list(index_uniques,:);
 
 iter_ = max(options_.periods,1);
 if M_.exo_nbr > 0
@@ -81,17 +76,19 @@ elseif options_.discretionary_policy
     end
     [oo_.dr,ys,info] = discretionary_policy_1(oo_,options_.instruments);
 else
-    if options_.logged_steady_state
+    if options_.logged_steady_state %if steady state was previously logged, undo this
         oo_.dr.ys=exp(oo_.dr.ys);
         oo_.steady_state=exp(oo_.steady_state);
+        options_.logged_steady_state=0;
     end
     [oo_.dr,info,M_,options_,oo_] = resol(0,M_,options_,oo_);
 end
 
-if options_.loglinear
-    oo_.dr.ys=log(oo_.dr.ys);
-    oo_.steady_state=log(oo_.steady_state);
-    options_old.logged_steady_state = 1;
+if options_.loglinear && isfield(oo_.dr,'ys') && options_.logged_steady_state==0 %log steady state for correct display of decision rule
+    oo_.dr.ys=log_variable(1:M_.endo_nbr,oo_.dr.ys,M_);
+    oo_.steady_state=log_variable(1:M_.endo_nbr,oo_.steady_state,M_);
+    options_old.logged_steady_state = 1; %make sure option is preserved outside of stoch_simul
+    options_.logged_steady_state=1; %set option for use in stoch_simul
 end
 if info(1)
     options_ = options_old;
@@ -112,14 +109,20 @@ if ~options_.noprint
     labels = deblank(M_.exo_names);
     headers = char('Variables',labels);
     lh = size(labels,2)+2;
-    dyntable(my_title,headers,labels,M_.Sigma_e,lh,10,6);
+    dyntable(options_,my_title,headers,labels,M_.Sigma_e,lh,10,6);
+    if options_.TeX
+        labels = deblank(M_.exo_names_tex);
+        headers = char('Variables',labels);
+        lh = size(labels,2)+2;
+        dyn_latex_table(M_,options_,my_title,'covar_ex_shocks',headers,labels,M_.Sigma_e,lh,10,6);
+    end
     if options_.partial_information
         skipline()
         disp('SOLUTION UNDER PARTIAL INFORMATION')
         skipline()
 
         if isfield(options_,'varobs')&& ~isempty(options_.varobs)
-            PCL_varobs=options_.varobs;
+            PCL_varobs=char(options_.varobs);
             disp('OBSERVED VARIABLES')
         else
             PCL_varobs=M_.endo_names;
@@ -148,11 +151,17 @@ if options_.periods > 0 && ~PI_PCL_solver
     if isempty(M_.endo_histval)
         y0 = oo_.dr.ys;
     else
-        y0 = M_.endo_histval;
+        if options_.loglinear
+            y0 = log_variable(1:M_.endo_nbr,M_.endo_histval,M_);
+        else
+            y0 = M_.endo_histval;
+        end
     end
     [ys, oo_] = simult(y0,oo_.dr,M_,options_,oo_);
     oo_.endo_simul = ys;
-    dyn2vec;
+    if ~options_.minimal_workspace
+        dyn2vec;
+    end
 end
 
 if options_.nomoments == 0
@@ -161,10 +170,10 @@ if options_.nomoments == 0
     elseif options_.periods == 0
         % There is no code for theoretical moments at 3rd order
         if options_.order <= 2
-            disp_th_moments(oo_.dr,var_list);
+            oo_=disp_th_moments(oo_.dr,var_list,M_,options_,oo_);
         end
     else
-        disp_moments(oo_.endo_simul,var_list);
+        oo_=disp_moments(oo_.endo_simul,var_list,M_,options_,oo_);
     end
 end
 
@@ -172,8 +181,8 @@ end
 if options_.irf
     var_listTeX = M_.endo_names_tex(i_var,:);
 
-    if TeX
-        fidTeX = fopen([M_.fname '_IRF.TeX'],'w');
+    if TeX && any(strcmp('eps',cellstr(options_.graph_format)))
+        fidTeX = fopen([M_.fname '_IRF.tex'],'w');
         fprintf(fidTeX,'%% TeX eps-loader file generated by stoch_simul.m (Dynare).\n');
         fprintf(fidTeX,['%% ' datestr(now,0) '\n']);
         fprintf(fidTeX,' \n');
@@ -190,8 +199,13 @@ if options_.irf
             if PI_PCL_solver
                 y=PCL_Part_info_irf (0, PCL_varobs, i_var, M_, oo_.dr, options_.irf, i);
             else
-                y=irf(oo_.dr,cs(M_.exo_names_orig_ord,i), options_.irf, options_.drop, ...
-                      options_.replic, options_.order);
+                if options_.order>1 && options_.relative_irf % normalize shock to 0.01 before IRF generation for GIRFs; multiply with 100 later
+                    y=irf(oo_.dr,cs(M_.exo_names_orig_ord,i)./cs(i,i)/100, options_.irf, options_.drop, ...
+                          options_.replic, options_.order);
+                else %for linear model, rescaling is done later
+                    y=irf(oo_.dr,cs(M_.exo_names_orig_ord,i), options_.irf, options_.drop, ...
+                          options_.replic, options_.order);
+                end
             end
             if ~options_.noprint && any(any(isnan(y))) && ~options_.pruning && ~(options_.order==1)
                 fprintf('\nstoch_simul:: The simulations conducted for generating IRFs to %s were explosive.\n',M_.exo_names(i,:))
@@ -200,7 +214,9 @@ if options_.irf
                 skipline(2);
             end
             if options_.relative_irf
-                y = 100*y/cs(i,i);
+                if options_.order==1 %multiply with 100 for backward compatibility
+                    y = 100*y/cs(i,i);
+                end
             end
             irfs   = [];
             mylist = [];
@@ -212,7 +228,7 @@ if options_.irf
                          y(i_var(j),:)');
                 eval(['oo_.irfs.' deblank(M_.endo_names(i_var(j),:)) '_' ...
                       deblank(M_.exo_names(i,:)) ' = y(i_var(j),:);']);
-                if max(abs(y(i_var(j),:))) > options_.impulse_responses.plot_threshold
+                if max(abs(y(i_var(j),:))) >= options_.impulse_responses.plot_threshold
                     irfs  = cat(1,irfs,y(i_var(j),:));
                     if isempty(mylist)
                         mylist = deblank(var_list(j,:));
@@ -238,10 +254,10 @@ if options_.irf
                 if nbplt == 0
                 elseif nbplt == 1
                     if options_.relative_irf
-                        hh = dyn_figure(options_,'Name',['Relative response to' ...
+                        hh = dyn_figure(options_.nodisplay,'Name',['Relative response to' ...
                                             ' orthogonalized shock to ' tit(i,:)]);
                     else
-                        hh = dyn_figure(options_,'Name',['Orthogonalized shock to' ...
+                        hh = dyn_figure(options_.nodisplay,'Name',['Orthogonalized shock to' ...
                                             ' ' tit(i,:)]);
                     end
                     for j = 1:number_of_plots_to_draw
@@ -251,17 +267,18 @@ if options_.irf
                         plot([1 options_.irf],[0 0],'-r','linewidth',0.5);
                         hold off
                         xlim([1 options_.irf]);
+                        remove_fractional_xticks;
                         title(deblank(mylist(j,:)),'Interpreter','none');
                     end
-                    dyn_saveas(hh,[M_.fname '_IRF_' deblank(tit(i,:))],options_);
-                    if TeX
+                    dyn_saveas(hh,[M_.fname '_IRF_' deblank(tit(i,:))],options_.nodisplay,options_.graph_format);
+                    if TeX && any(strcmp('eps',cellstr(options_.graph_format)))
                         fprintf(fidTeX,'\\begin{figure}[H]\n');
                         for j = 1:number_of_plots_to_draw
                             fprintf(fidTeX,['\\psfrag{%s}[1][][0.5][0]{$%s$}\n'],deblank(mylist(j,:)),deblank(mylistTeX(j,:)));
                         end
                         fprintf(fidTeX,'\\centering \n');
-                        fprintf(fidTeX,'\\includegraphics[scale=0.5]{%s_IRF_%s}\n',M_.fname,deblank(tit(i,:)));
-                        fprintf(fidTeX,'\\caption{Impulse response functions (orthogonalized shock to $%s$).}',titTeX(i,:));
+                        fprintf(fidTeX,'\\includegraphics[width=%2.2f\\textwidth]{%s_IRF_%s}\n',options_.figures.textwidth*min(j/nc,1),M_.fname,deblank(tit(i,:)));
+                        fprintf(fidTeX,'\\caption{Impulse response functions (orthogonalized shock to $%s$).}\n',titTeX(i,:));
                         fprintf(fidTeX,'\\label{Fig:IRF:%s}\n',deblank(tit(i,:)));
                         fprintf(fidTeX,'\\end{figure}\n');
                         fprintf(fidTeX,' \n');
@@ -269,10 +286,10 @@ if options_.irf
                 else
                     for fig = 1:nbplt-1
                         if options_.relative_irf
-                            hh = dyn_figure(options_,'Name',['Relative response to orthogonalized shock' ...
+                            hh = dyn_figure(options_.nodisplay,'Name',['Relative response to orthogonalized shock' ...
                                                 ' to ' tit(i,:) ' figure ' int2str(fig)]);
                         else
-                            hh = dyn_figure(options_,'Name',['Orthogonalized shock to ' tit(i,:) ...
+                            hh = dyn_figure(options_.nodisplay,'Name',['Orthogonalized shock to ' tit(i,:) ...
                                                 ' figure ' int2str(fig)]);
                         end
                         for plt = 1:nstar
@@ -282,16 +299,17 @@ if options_.irf
                             plot([1 options_.irf],[0 0],'-r','linewidth',0.5);
                             hold off
                             xlim([1 options_.irf]);
+                            remove_fractional_xticks
                             title(deblank(mylist((fig-1)*nstar+plt,:)),'Interpreter','none');
                         end
-                        dyn_saveas(hh,[ M_.fname '_IRF_' deblank(tit(i,:)) int2str(fig)],options_);
-                        if TeX
+                        dyn_saveas(hh,[ M_.fname '_IRF_' deblank(tit(i,:)) int2str(fig)],options_.nodisplay,options_.graph_format);
+                        if TeX && any(strcmp('eps',cellstr(options_.graph_format)))
                             fprintf(fidTeX,'\\begin{figure}[H]\n');
                             for j = 1:nstar
                                 fprintf(fidTeX,['\\psfrag{%s}[1][][0.5][0]{$%s$}\n'],deblank(mylist((fig-1)*nstar+j,:)),deblank(mylistTeX((fig-1)*nstar+j,:)));
                             end
                             fprintf(fidTeX,'\\centering \n');
-                            fprintf(fidTeX,'\\includegraphics[scale=0.5]{%s_IRF_%s%s}\n',M_.fname,deblank(tit(i,:)),int2str(fig));
+                            fprintf(fidTeX,'\\includegraphics[width=%2.2f\\textwidth]{%s_IRF_%s%s}\n',options_.figures.textwidth*min(plt/nc,1),M_.fname,deblank(tit(i,:)),int2str(fig));
                             if options_.relative_irf
                                 fprintf(fidTeX,['\\caption{Relative impulse response' ...
                                                 ' functions (orthogonalized shock to $%s$).}'],deblank(titTeX(i,:)));
@@ -299,14 +317,14 @@ if options_.irf
                                 fprintf(fidTeX,['\\caption{Impulse response functions' ...
                                                 ' (orthogonalized shock to $%s$).}'],deblank(titTeX(i,:)));
                             end
-                            fprintf(fidTeX,'\\label{Fig:BayesianIRF:%s:%s}\n',deblank(tit(i,:)),int2str(fig));
+                            fprintf(fidTeX,'\\label{Fig:IRF:%s:%s}\n',deblank(tit(i,:)),int2str(fig));
                             fprintf(fidTeX,'\\end{figure}\n');
                             fprintf(fidTeX,' \n');
                         end
                     end
-                    hh = dyn_figure(options_,'Name',['Orthogonalized shock to ' tit(i,:) ' figure ' int2str(nbplt) '.']);
+                    hh = dyn_figure(options_.nodisplay,'Name',['Orthogonalized shock to ' tit(i,:) ' figure ' int2str(nbplt) '.']);
                     m = 0;
-                    for plt = 1:number_of_plots_to_draw-(nbplt-1)*nstar;
+                    for plt = 1:number_of_plots_to_draw-(nbplt-1)*nstar
                         m = m+1;
                         subplot(lr,lc,m);
                         plot(1:options_.irf,transpose(irfs((nbplt-1)*nstar+plt,:)),'-k','linewidth',1);
@@ -314,16 +332,17 @@ if options_.irf
                         plot([1 options_.irf],[0 0],'-r','linewidth',0.5);
                         hold off
                         xlim([1 options_.irf]);
+                        remove_fractional_xticks
                         title(deblank(mylist((nbplt-1)*nstar+plt,:)),'Interpreter','none');
                     end
-                    dyn_saveas(hh,[ M_.fname '_IRF_' deblank(tit(i,:)) int2str(nbplt) ],options_);
-                    if TeX
+                    dyn_saveas(hh,[ M_.fname '_IRF_' deblank(tit(i,:)) int2str(nbplt) ],options_.nodisplay,options_.graph_format);
+                    if TeX && any(strcmp('eps',cellstr(options_.graph_format)))
                         fprintf(fidTeX,'\\begin{figure}[H]\n');
                         for j = 1:m
                             fprintf(fidTeX,['\\psfrag{%s}[1][][0.5][0]{$%s$}\n'],deblank(mylist((nbplt-1)*nstar+j,:)),deblank(mylistTeX((nbplt-1)*nstar+j,:)));
                         end
                         fprintf(fidTeX,'\\centering \n');
-                        fprintf(fidTeX,'\\includegraphics[scale=0.5]{%s_IRF_%s%s}\n',M_.fname,deblank(tit(i,:)),int2str(nbplt));
+                        fprintf(fidTeX,'\\includegraphics[width=%2.2f\\textwidth]{%s_IRF_%s%s}\n',options_.figures.textwidth*min(m/lc,1),M_.fname,deblank(tit(i,:)),int2str(nbplt));
                         if options_.relative_irf
                             fprintf(fidTeX,['\\caption{Relative impulse response functions' ...
                                             ' (orthogonalized shock to $%s$).}'],deblank(titTeX(i,:)));
@@ -339,7 +358,7 @@ if options_.irf
             end
         end
     end
-    if TeX
+    if TeX && any(strcmp('eps',cellstr(options_.graph_format)))
         fprintf(fidTeX,' \n');
         fprintf(fidTeX,'%% End Of TeX file. \n');
         fclose(fidTeX);
@@ -347,7 +366,7 @@ if options_.irf
 end
 
 if options_.SpectralDensity.trigger == 1
-    [omega,f] = UnivariateSpectralDensity(oo_.dr,var_list);
+    [oo_] = UnivariateSpectralDensity(M_,oo_,options_,var_list);
 end
 
 
