@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 Dynare Team
+ * Copyright (C) 2003-2017 Dynare Team
  *
  * This file is part of Dynare.
  *
@@ -41,13 +41,36 @@ InitParamStatement::checkPass(ModFileStructure &mod_file_struct, WarningConsolid
 }
 
 void
-InitParamStatement::writeOutput(ostream &output, const string &basename) const
+InitParamStatement::writeOutput(ostream &output, const string &basename, bool minimal_workspace) const
 {
   int id = symbol_table.getTypeSpecificID(symb_id) + 1;
   output << "M_.params( " << id << " ) = ";
   param_value->writeOutput(output);
   output << ";" << endl;
-  output << symbol_table.getName(symb_id) << " = M_.params( " << id << " );\n";
+  if (!minimal_workspace)
+    output << symbol_table.getName(symb_id) << " = M_.params( " << id << " );" << endl;
+}
+
+void
+InitParamStatement::writeJuliaOutput(ostream &output, const string &basename)
+{
+  int id = symbol_table.getTypeSpecificID(symb_id) + 1;
+  output << "model_.params[ " << id << " ] = ";
+  param_value->writeOutput(output);
+  output << endl;
+  // Do we really need this?
+  // if (!minimal_workspace)
+  //   output << symbol_table.getName(symb_id) << " = model_.params[ " << id << " ]" << endl;
+}
+
+void
+InitParamStatement::writeCOutput(ostream &output, const string &basename)
+{
+  int id = symbol_table.getTypeSpecificID(symb_id);
+  output << "params[ " << id << " ] = ";
+  param_value->writeOutput(output);
+  output << ";" << endl;
+  output << "double " << symbol_table.getName(symb_id) << " = params[ " << id << " ];" << endl;
 }
 
 void
@@ -176,7 +199,7 @@ InitValStatement::checkPass(ModFileStructure &mod_file_struct, WarningConsolidat
 }
 
 void
-InitValStatement::writeOutput(ostream &output, const string &basename) const
+InitValStatement::writeOutput(ostream &output, const string &basename, bool minimal_workspace) const
 {
   output << "%" << endl
          << "% INITVAL instructions" << endl
@@ -190,12 +213,12 @@ InitValStatement::writeOutput(ostream &output, const string &basename) const
 void
 InitValStatement::writeOutputPostInit(ostream &output) const
 {
-  output << "if M_.exo_nbr > 0;" << endl
-         << "\too_.exo_simul = [ones(M_.maximum_lag,1)*oo_.exo_steady_state'];" << endl
-         <<"end;" << endl
-         << "if M_.exo_det_nbr > 0;" << endl
-         << "\too_.exo_det_simul = [ones(M_.maximum_lag,1)*oo_.exo_det_steady_state'];" << endl
-         <<"end;" << endl;
+  output << "if M_.exo_nbr > 0" << endl
+         << "\too_.exo_simul = ones(M_.maximum_lag,1)*oo_.exo_steady_state';" << endl
+         <<"end" << endl
+         << "if M_.exo_det_nbr > 0" << endl
+         << "\too_.exo_det_simul = ones(M_.maximum_lag,1)*oo_.exo_det_steady_state';" << endl
+         <<"end" << endl;
 }
 
 EndValStatement::EndValStatement(const init_values_t &init_values_arg,
@@ -229,16 +252,10 @@ EndValStatement::checkPass(ModFileStructure &mod_file_struct, WarningConsolidati
 
   if (endogs.size() > 0 || exogs.size() > 0)
     exit(EXIT_FAILURE);
-
-  if (mod_file_struct.shocks_present_but_simul_not_yet)
-    {
-      cerr << "ERROR: Putting a \"shocks\" block before an \"endval\" block is not permitted. Please swap the two blocks. This limitation will be removed in a future release of Dynare." << endl;
-      exit(EXIT_FAILURE);
-    }
 }
 
 void
-EndValStatement::writeOutput(ostream &output, const string &basename) const
+EndValStatement::writeOutput(ostream &output, const string &basename, bool minimal_workspace) const
 {
   output << "%" << endl
          << "% ENDVAL instructions" << endl
@@ -251,25 +268,65 @@ EndValStatement::writeOutput(ostream &output, const string &basename) const
 }
 
 HistValStatement::HistValStatement(const hist_values_t &hist_values_arg,
-                                   const SymbolTable &symbol_table_arg) :
+                                   const SymbolTable &symbol_table_arg,
+                                   const bool &all_values_required_arg) :
   hist_values(hist_values_arg),
-  symbol_table(symbol_table_arg)
+  symbol_table(symbol_table_arg),
+  all_values_required(all_values_required_arg)
 {
 }
 
 void
 HistValStatement::checkPass(ModFileStructure &mod_file_struct, WarningConsolidation &warnings)
 {
-  mod_file_struct.histval_present = true;
+  if (all_values_required)
+    {
+      set<int> unused_endo = symbol_table.getEndogenous();
+      set<int> unused_exo = symbol_table.getExogenous();
+
+      set<int>::iterator sit;
+      for (hist_values_t::const_iterator it = hist_values.begin();
+           it != hist_values.end(); it++)
+        {
+          sit = unused_endo.find(it->first.first);
+          if (sit != unused_endo.end())
+            unused_endo.erase(sit);
+
+          sit = unused_exo.find(it->first.first);
+          if (sit != unused_exo.end())
+            unused_exo.erase(sit);
+        }
+
+      if (unused_endo.size() > 0)
+        {
+          cerr << "ERROR: You have not set the following endogenous variables in histval:";
+          for (set<int>::const_iterator it = unused_endo.begin(); it != unused_endo.end(); it++)
+            cerr << " " << symbol_table.getName(*it);
+          cerr << endl;
+        }
+
+      if (unused_exo.size() > 0)
+        {
+          cerr << "ERROR: You have not set the following exogenous variables in endval:";
+          for (set<int>::const_iterator it = unused_exo.begin(); it != unused_exo.end(); it++)
+            cerr << " " << symbol_table.getName(*it);
+          cerr << endl;
+        }
+
+      if (unused_endo.size() > 0 || unused_exo.size() > 0)
+        exit(EXIT_FAILURE);
+    }
 }
 
 void
-HistValStatement::writeOutput(ostream &output, const string &basename) const
+HistValStatement::writeOutput(ostream &output, const string &basename, bool minimal_workspace) const
 {
   output << "%" << endl
          << "% HISTVAL instructions" << endl
          << "%" << endl
-         << "M_.endo_histval = zeros(M_.endo_nbr,M_.maximum_endo_lag);" << endl;
+         << "M_.endo_histval = zeros(M_.endo_nbr,M_.maximum_lag);" << endl
+         << "M_.exo_histval = zeros(M_.exo_nbr,M_.maximum_lag);" << endl
+         << "M_.exo_det_histval = zeros(M_.exo_det_nbr,M_.maximum_lag);" << endl;
 
   for (hist_values_t::const_iterator it = hist_values.begin();
        it != hist_values.end(); it++)
@@ -305,11 +362,11 @@ HistValStatement::writeOutput(ostream &output, const string &basename) const
       int tsid = symbol_table.getTypeSpecificID(symb_id) + 1;
 
       if (type == eEndogenous)
-        output << "M_.endo_histval( " << tsid << ", M_.maximum_endo_lag + " << lag << ") = ";
+        output << "M_.endo_histval( " << tsid << ", M_.maximum_lag + " << lag << ") = ";
       else if (type == eExogenous)
-        output << "oo_.exo_simul( M_.maximum_lag + " << lag << ", " << tsid << " ) = ";
-      else if (type != eExogenousDet)
-        output << "oo_.exo_det_simul( M_.maximum_lag + " << lag  << ", " << tsid << " ) = ";
+        output << "M_.exo_histval( " << tsid << ", M_.maximum_lag + " << lag << ") = ";
+      else if (type == eExogenousDet)
+        output << "M_.exo_det_histval( " << tsid << ", M_.maximum_lag + " << lag << ") = ";
 
       expression->writeOutput(output);
       output << ";" << endl;
@@ -322,7 +379,7 @@ InitvalFileStatement::InitvalFileStatement(const string &filename_arg) :
 }
 
 void
-InitvalFileStatement::writeOutput(ostream &output, const string &basename) const
+InitvalFileStatement::writeOutput(ostream &output, const string &basename, bool minimal_workspace) const
 {
   output << "%" << endl
          << "% INITVAL_FILE statement" << endl
@@ -331,6 +388,17 @@ InitvalFileStatement::writeOutput(ostream &output, const string &basename) const
          << "initvalf('" << filename << "');" << endl;
 }
 
+HistvalFileStatement::HistvalFileStatement(const string &filename_arg) :
+  filename(filename_arg)
+{
+}
+
+void
+HistvalFileStatement::writeOutput(ostream &output, const string &basename, bool minimal_workspace) const
+{
+  output << "histvalf('" << filename << "');" << endl;
+}
+
 HomotopyStatement::HomotopyStatement(const homotopy_values_t &homotopy_values_arg,
                                      const SymbolTable &symbol_table_arg) :
   homotopy_values(homotopy_values_arg),
@@ -339,7 +407,7 @@ HomotopyStatement::HomotopyStatement(const homotopy_values_t &homotopy_values_ar
 }
 
 void
-HomotopyStatement::writeOutput(ostream &output, const string &basename) const
+HomotopyStatement::writeOutput(ostream &output, const string &basename, bool minimal_workspace) const
 {
   output << "%" << endl
          << "% HOMOTOPY_SETUP instructions" << endl
@@ -373,7 +441,7 @@ SaveParamsAndSteadyStateStatement::SaveParamsAndSteadyStateStatement(const strin
 }
 
 void
-SaveParamsAndSteadyStateStatement::writeOutput(ostream &output, const string &basename) const
+SaveParamsAndSteadyStateStatement::writeOutput(ostream &output, const string &basename, bool minimal_workspace) const
 {
   output << "save_params_and_steady_state('" << filename << "');" << endl;
 }
@@ -414,7 +482,7 @@ LoadParamsAndSteadyStateStatement::LoadParamsAndSteadyStateStatement(const strin
 }
 
 void
-LoadParamsAndSteadyStateStatement::writeOutput(ostream &output, const string &basename) const
+LoadParamsAndSteadyStateStatement::writeOutput(ostream &output, const string &basename, bool minimal_workspace) const
 {
   for (map<int, string>::const_iterator it = content.begin();
        it != content.end(); it++)
